/*
 * Copyright 2010 James Geboski <jgeboski@users.sourceforge.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef LIBG19DRAW_H
#define LIBG19DRAW_H

#include <string.h>

#ifdef FT2_SUPPORT

#include <ft2build.h>
#include FT_FREETYPE_H
#include FT_BITMAP_H

#endif

#ifdef __cplusplus
extern "C"
{
#endif

#define LIBG19DRAW_VERSION_MAJOR	1
#define LIBG19DRAW_VERSION_MINOR	1
#define LIBG19DRAW_VERSION_MICRO	0

#define LIBG19DRAW_VERSION			"1.1.0"

#define G19_WIDTH		320
#define G19_HEIGHT		240
#define G19_PIXEL_SIZE	2

#define G19ColorA(r, g, b, a) ((unsigned int) r | g << 8 | b << 16 | a << 24)
#define G19Color(r, g, b) ((unsigned int) G19ColorA(r, g, b, 0))

#define G19SetA(c, a) ((unsigned int) c | a << 24)

#define G19R(c) ((unsigned char) c)
#define G19G(c) ((unsigned char) (c >> 8))
#define G19B(c) ((unsigned char) (c >> 16))
#define G19A(c) ((unsigned char) (c >> 24))

typedef struct
{
	int width;
	int height;
	size_t size;
	unsigned char * data;
}
G19Draw;

#ifdef FT2_SUPPORT

typedef struct
{
	FT_Library lib;
	FT_Face face;
	int width;
	int height;
	int space;
	unsigned int color;
}
G19Font;

#endif

/**
 * Creates a new G19Draw
 * 
 * @param width width in pixels of image to create
 * @param height height in pixels of image to create
 * 
 * @return pointer to a G19Draw
 **/
G19Draw * g19_draw_new(int width, int height);

/**
 * Creates a new G19Draw from an image
 * The image must me a 24-bit or 32-bit bitmap with no compression
 * 
 * @param path path to the image to load
 * 
 * @return pointer to a G19Draw; On failure NULL
 **/
G19Draw * g19_draw_bmp(char * path);

/**
 * Frees up up allocated memory the G19Draw used
 * 
 * @param draw pointer to the G19Draw to free
 * 
 * @return pointer to a G19Draw
 **/
void g19_draw_free(G19Draw * draw);

/**
 * Save a G19Draw to an 24-bit bitmap
 * 
 * @param draw pointer to a G19Draw
 * @param path path to save the image to
 * 
 * @return 0 on success; non-zero on error
 **/
int g19_draw_to_bmp(G19Draw * draw, char * path);

/**
 * Resizes a G19Draw
 * 
 * @param width width in pixels to resize to
 * @param height height in pixels to resize to
 * @param color the base color of the new image; G19Color() or G19ColorA() can be used
 * @param free_old free the allocated memory of the old old G19Draw; 0 for no, 1 for yes
 * 
 * @return a pointer to the new G19Draw or NULL on error
 **/
G19Draw * g19_draw_resize(G19Draw * draw, int width, int height, unsigned int color, int free_old);

/**
 * Insert a G19Draw onto another G19Draw
 * 
 * @param draw pointer to the destination G19Draw
 * @param idraw pointer to the source G19Draw
 * @param ix x coordinate of the upper-left
 * @param iy y coordinate of the upper-left
 **/
void g19_draw_insert(G19Draw * draw, G19Draw * idraw, int ix, int iy);

/**
 * Insert a G19Draw onto another G19Draw with alpha blending
 * 
 * @param draw pointer to the destination G19Draw
 * @param idraw pointer to the source G19Draw
 * @param ix x coordinate of the upper-left
 * @param iy y coordinate of the upper-left
 * @param opac if less that 0 then it will use the values stores or specify 0 - 255
 **/
void g19_draw_insert_ab(G19Draw * draw, G19Draw * idraw, int ix, int iy, int opac);

/**
 * Gets the pixel value at the specified coordinates
 * 
 * @param draw a pointer G19Draw
 * @param x x coordinate of pixel
 * @param y y coordinate of pixel
 * 
 * @return an unsigned int that can be used in unison with: G19R(), G19G(), G19B(), G19A()
 **/
unsigned int g19_draw_get_pixel(G19Draw * draw, int x, int y);

/**
 * Sets the pixel value at the specified coordinates
 * 
 * @param draw a pointer G19Draw
 * @param x x coordinate of pixel
 * @param y y coordinate of pixel
 * @param color an unsigned int that hold the color; G19Color() or G19ColorA() can be used
 **/
void g19_draw_set_pixel(G19Draw * draw, int x, int y, unsigned int color);

/**
 * Fills the G19Draw with a color
 * 
 * @param draw a pointer G19Draw
 * @param color an unsigned int that hold the color; G19Color() or G19ColorA() can be used
 **/
void g19_draw_fill(G19Draw * draw, unsigned int color);

/**
 * Draws a line on the G19Draw
 * 
 * @param draw a pointer G19Draw
 * @param x1 x coordinate of the first point
 * @param y1 y coordinate of the first point
 * @param x2 x coordinate of the second point
 * @param y2 y coordinate of the second point
 * @param color an unsigned int that hold the color; G19Color() or G19ColorA() can be used
 **/
void g19_draw_line(G19Draw * draw, int x1, int y1, int x2, int y2, unsigned int color);

/**
 * Draws a rectangle on the G19Draw
 * 
 * @param draw a pointer G19Draw
 * @param left left most point int terms of x
 * @param top top most point int terms of y
 * @param right most point int terms of x
 * @param bottom most point int terms of y
 * @param fill 0 for 1 pixel border or 1 to fill the rectangle
 * @param color an unsigned int that hold the color; G19Color() or G19ColorA() can be used
 **/
void g19_draw_rect(G19Draw * draw, int left, int top, int right, int bottom, int fill, unsigned int color);

/**
 * Draws a circle on the G19Draw
 * 
 * @param draw a pointer G19Draw
 * @param cx x coordinate of the circle's center
 * @param cy y coordinate of the circle's center
 * @param cr radius of the circle
 * @param fill 0 for 1 pixel border or 1 to fill the rectangle
 * @param color an unsigned int that hold the color; G19Color() or G19ColorA() can be used
 **/
void g19_draw_circle(G19Draw * draw, int cx, int cy, int cr, int fill, unsigned int color);


#ifdef FT2_SUPPORT

/**
 * Creates a new G19Font
 * 
 * @param ttf_path path to the ttf file
 * @param width width of text
 * @param height height of text
 * @param space space between chars in pixels
 * @param color an unsigned int that hold the color; G19Color() or G19ColorA() can be used
 * 
 * @return pointer to a G19Font or NULL on error
 **/
G19Font * g19_font_new(char * ttf_path, int width, int height, int space, unsigned int color);

/**
 * Sets the character size on the G19Font
 * 
 * @param font a pointer G19Font
 * @param width width of text
 * @param height height of text
 **/
void g19_font_set_size(G19Font * font, int width, int height);

/**
 * Sets the character spacing on the G19Font
 * 
 * @param font a pointer G19Font
 * @param space between characters in pixels
 **/
void g19_font_set_space(G19Font * font, int space);

/**
 * Sets the color on the G19Font
 * 
 * @param font a pointer G19Font
 * @param color the color to set the text; G19Color() or G19ColorA() can be used
 **/
void g19_font_set_color(G19Font * font, unsigned int color);

/**
 * Draws text on the G19Draw with G19Font
 * 
 * @param font a pointer G19Font
 * @param draw a pointer G19Draw
 * @param x x coordinate of the upper-left
 * @param y y coordinate of the upper-left
 * @param text pointer to an array of chars to be drawn
 * 
 * @return the number of characters written or < 0 on error
 **/
int g19_font_draw(G19Font * font, G19Draw * draw, int x, int y, char * text);


/**
 * Frees a G19Font
 * 
 * @param font a pointer G19Font
 **/
void g19_font_free(G19Font * font);

#endif

#ifdef __cplusplus
}
#endif

#endif

